<?php

namespace SilverStripe\Forms\GridField;

use SilverStripe\Control\Controller;
use SilverStripe\View\ArrayData;
use SilverStripe\View\SSViewer;

/**
 * A button that allows a user to view readonly details of a record. This is
 * disabled by default and intended for use in readonly {@link GridField}
 * instances.
 */
class GridFieldViewButton extends AbstractGridFieldComponent implements GridField_ColumnProvider, GridField_ActionMenuLink
{
    private bool $suffixViewToUrl = true;

    public function getSuffixViewToUrl(): bool
    {
        return $this->suffixViewToUrl;
    }

    public function setSuffixViewToUrl(bool $suffixViewToUrl): static
    {
        $this->suffixViewToUrl = $suffixViewToUrl;
        return $this;
    }

    /**
     * @inheritdoc
     */
    public function getTitle($gridField, $record, $columnName)
    {
        return _t(__CLASS__ . '.VIEW', "View");
    }

    /**
     * @inheritdoc
     */
    public function getGroup($gridField, $record, $columnName)
    {
        return GridField_ActionMenuItem::DEFAULT_GROUP;
    }

    /**
     * @inheritdoc
     */
    public function getExtraData($gridField, $record, $columnName)
    {
        return [
            "classNames" => "font-icon-eye action-detail view-link"
        ];
    }

    /**
     * @inheritdoc
     */
    public function getUrl($gridField, $record, $columnName)
    {
        $parts = [$gridField->Link('item'), $record->ID];
        if ($this->getSuffixViewToUrl()) {
            $parts[] = 'view';
        }
        $link = Controller::join_links(...$parts);
        return $gridField->addAllStateToUrl($link);
    }

    public function augmentColumns($field, &$columns)
    {
        if (!in_array('Actions', $columns ?? [])) {
            $columns[] = 'Actions';
        }
    }

    public function getColumnsHandled($field)
    {
        return ['Actions'];
    }

    public function getColumnContent($field, $record, $col)
    {
        // Assume item can be viewed if canView() isn't implemented
        if ($record->hasMethod('canView') && !$record->canView()) {
            return null;
        }
        $data = new ArrayData([
            'Link' => $this->getURL($field, $record, $col),
        ]);
        $template = SSViewer::get_templates_by_class($this, '', __CLASS__);
        return $data->renderWith($template);
    }

    public function getColumnAttributes($field, $record, $col)
    {
        return ['class' => 'grid-field__col-compact'];
    }

    public function getColumnMetadata($gridField, $col)
    {
        return ['title' => null];
    }
}
